@@ -2,12 +2,13 @@
  * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
  * SPDX-License-Identifier: Apache-2.0.
  */
+#include <aws/cal/private/ecc.h>
+
+#include <aws/cal/cal.h>
+#include <aws/cal/private/der.h>
+
 #include <Security/SecKey.h>
 #include <Security/Security.h>
-#include <aws/cal/cal.h>
-#include <aws/cal/ecc.h>
-#include <aws/cal/private/der.h>
-#include <aws/cal/private/ecc.h>
 
 #if !defined(AWS_OS_IOS)
 #    include <Security/SecSignVerifyTransform.h>
@@ -207,7 +208,7 @@ error:
     return NULL;
 }
 
-struct aws_ecc_key_pair *aws_ecc_key_pair_new_from_private_key(
+struct aws_ecc_key_pair *aws_ecc_key_pair_new_from_private_key_impl(
     struct aws_allocator *allocator,
     enum aws_ecc_curve_name curve_name,
     const struct aws_byte_cursor *priv_key) {
@@ -270,7 +271,7 @@ error:
     return NULL;
 }
 
-struct aws_ecc_key_pair *aws_ecc_key_pair_new_from_public_key(
+struct aws_ecc_key_pair *aws_ecc_key_pair_new_from_public_key_impl(
     struct aws_allocator *allocator,
     enum aws_ecc_curve_name curve_name,
     const struct aws_byte_cursor *public_key_x,
