@@ -5,6 +5,7 @@ import subprocess
 import sys
 import urllib.request
 import unittest
+import os.path
 
 TIMEOUT = 100
 
@@ -14,6 +15,13 @@ if not elasticurl_cmd_prefix:
     print('You must pass the elasticurl cmd prefix')
     sys.exit(-1)
 
+program_to_run = elasticurl_cmd_prefix[0]
+
+if 'bin' in program_to_run:
+    if not os.path.exists(program_to_run):
+        print('the program_to_run is not found, skip integration test')
+        sys.exit(0)
+
 # Remove args from sys.argv so that unittest doesn't also try to parse them.
 sys.argv = sys.argv[:1]
 
