@@ -31,6 +31,15 @@ static struct aws_http_connection_system_vtable s_default_system_vtable = {
 
 static const struct aws_http_connection_system_vtable *s_system_vtable_ptr = &s_default_system_vtable;
 
+void aws_http_client_bootstrap_destroy(struct aws_http_client_bootstrap *bootstrap) {
+    /* During allocating, the underlying stuctures should be allocated with the bootstrap by aws_mem_acquire_many. Thus,
+     * we only need to clean up the first pointer which is the bootstrap */
+    if (bootstrap->alpn_string_map) {
+        aws_hash_table_clean_up(bootstrap->alpn_string_map);
+    }
+    aws_mem_release(bootstrap->alloc, bootstrap);
+}
+
 void aws_http_connection_set_system_vtable(const struct aws_http_connection_system_vtable *system_vtable) {
     s_system_vtable_ptr = system_vtable;
 }
@@ -76,7 +85,9 @@ struct aws_http_connection *aws_http_connection_new_channel_handler(
     bool is_server,
     bool is_using_tls,
     bool manual_window_management,
+    bool prior_knowledge_http2,
     size_t initial_window_size,
+    const struct aws_hash_table *alpn_string_map,
     const struct aws_http1_connection_options *http1_options,
     const struct aws_http2_connection_options *http2_options) {
 
@@ -123,7 +134,32 @@ struct aws_http_connection *aws_http_connection_new_channel_handler(
         struct aws_channel_handler *tls_handler = tls_slot->handler;
         struct aws_byte_buf protocol = aws_tls_handler_protocol(tls_handler);
         if (protocol.len) {
-            if (aws_string_eq_byte_buf(s_alpn_protocol_http_1_1, &protocol)) {
+            bool customized = false;
+            if (alpn_string_map) {
+                customized = true;
+                struct aws_string *negotiated_result = aws_string_new_from_buf(alloc, &protocol);
+                struct aws_hash_element *found = NULL;
+                aws_hash_table_find(alpn_string_map, (void *)negotiated_result, &found);
+                if (found) {
+                    version = (enum aws_http_version)(size_t)found->value;
+                    AWS_LOGF_DEBUG(
+                        AWS_LS_HTTP_CONNECTION,
+                        "static: Customized ALPN protocol " PRInSTR " used. " PRInSTR " client connection established.",
+                        AWS_BYTE_BUF_PRI(protocol),
+                        AWS_BYTE_CURSOR_PRI(aws_http_version_to_str(version)));
+                } else {
+                    AWS_LOGF_ERROR(
+                        AWS_LS_HTTP_CONNECTION,
+                        "static: Customized ALPN protocol " PRInSTR
+                        " used. However the it's not found in the ALPN map provided.",
+                        AWS_BYTE_BUF_PRI(protocol));
+                    version = AWS_HTTP_VERSION_UNKNOWN;
+                }
+                aws_string_destroy(negotiated_result);
+            }
+            if (customized) {
+                /* Do nothing */
+            } else if (aws_string_eq_byte_buf(s_alpn_protocol_http_1_1, &protocol)) {
                 version = AWS_HTTP_VERSION_1_1;
             } else if (aws_string_eq_byte_buf(s_alpn_protocol_http_2, &protocol)) {
                 version = AWS_HTTP_VERSION_2;
@@ -135,6 +171,10 @@ struct aws_http_connection *aws_http_connection_new_channel_handler(
                 version = AWS_HTTP_VERSION_1_1;
             }
         }
+    } else {
+        if (prior_knowledge_http2) {
+            version = AWS_HTTP_VERSION_2;
+        }
     }
 
     /* Create connection/handler */
@@ -345,6 +385,29 @@ struct aws_channel *aws_http_connection_get_channel(struct aws_http_connection *
     return connection->channel_slot->channel;
 }
 
+int aws_http_alpn_map_init(struct aws_allocator *allocator, struct aws_hash_table *map) {
+    AWS_ASSERT(allocator);
+    AWS_ASSERT(map);
+    int result = aws_hash_table_init(
+        map,
+        allocator,
+        5 /* initial size */,
+        aws_hash_string,
+        aws_hash_callback_string_eq,
+        aws_hash_callback_string_destroy,
+        NULL);
+    if (result) {
+        /* OOM will crash */
+        int error_code = aws_last_error();
+        AWS_LOGF_ERROR(
+            AWS_LS_HTTP_CONNECTION,
+            "Failed to initialize ALPN map with error code %d (%s)",
+            error_code,
+            aws_error_name(error_code));
+    }
+    return result;
+}
+
 void aws_http_connection_acquire(struct aws_http_connection *connection) {
     AWS_ASSERT(connection);
     aws_atomic_fetch_add(&connection->refcount, 1);
@@ -400,15 +463,19 @@ static void s_server_bootstrap_on_accept_channel_setup(
     }
     /* Create connection */
     /* TODO: expose http1/2 options to server API */
-    struct aws_http1_connection_options http1_options = AWS_HTTP1_CONNECTION_OPTIONS_INIT;
-    struct aws_http2_connection_options http2_options = AWS_HTTP2_CONNECTION_OPTIONS_INIT;
+    struct aws_http1_connection_options http1_options;
+    AWS_ZERO_STRUCT(http1_options);
+    struct aws_http2_connection_options http2_options;
+    AWS_ZERO_STRUCT(http2_options);
     connection = aws_http_connection_new_channel_handler(
         server->alloc,
         channel,
         true,
         server->is_using_tls,
         server->manual_window_management,
+        false, /* prior_knowledge_http2 */
         server->initial_window_size,
+        NULL, /* alpn_string_map */
         &http1_options,
         &http2_options);
     if (!connection) {
@@ -727,7 +794,7 @@ static void s_client_bootstrap_on_channel_setup(
         http_bootstrap->on_setup(NULL, error_code, http_bootstrap->user_data);
 
         /* Clean up the http_bootstrap, it has no more work to do. */
-        aws_mem_release(http_bootstrap->alloc, http_bootstrap);
+        aws_http_client_bootstrap_destroy(http_bootstrap);
         return;
     }
 
@@ -739,7 +806,9 @@ static void s_client_bootstrap_on_channel_setup(
         false,
         http_bootstrap->is_using_tls,
         http_bootstrap->manual_window_management,
+        http_bootstrap->prior_knowledge_http2,
         http_bootstrap->initial_window_size,
+        http_bootstrap->alpn_string_map,
         &http_bootstrap->http1_options,
         &http_bootstrap->http2_options);
     if (!http_bootstrap->connection) {
@@ -831,7 +900,7 @@ static void s_client_bootstrap_on_channel_shutdown(
     }
 
     /* Clean up bootstrapper */
-    aws_mem_release(http_bootstrap->alloc, http_bootstrap);
+    aws_http_client_bootstrap_destroy(http_bootstrap);
 }
 
 static int s_validate_http_client_connection_options(const struct aws_http_client_connection_options *options) {
@@ -876,6 +945,61 @@ static int s_validate_http_client_connection_options(const struct aws_http_clien
     return AWS_OP_SUCCESS;
 }
 
+struct s_copy_alpn_string_map_context {
+    struct aws_hash_table *map;
+    struct aws_allocator *allocator;
+};
+
+/* put every item into the source to make a deep copy of the map */
+static int s_copy_alpn_string_map(void *context, struct aws_hash_element *item) {
+    struct s_copy_alpn_string_map_context *func_context = context;
+    struct aws_hash_table *dest = func_context->map;
+    /* make a deep copy of the string and hash map will own the copy */
+    struct aws_string *key_copy = aws_string_new_from_string(func_context->allocator, item->key);
+    int was_created;
+    if (aws_hash_table_put(dest, key_copy, item->value, &was_created)) {
+        int error_code = aws_last_error();
+        AWS_LOGF_ERROR(
+            AWS_LS_HTTP_CONNECTION,
+            "Failed to copy ALPN map with error code %d (%s)",
+            error_code,
+            aws_error_name(error_code));
+        /* failed to put into the table, we need to clean up the copy ourselves */
+        aws_string_destroy(key_copy);
+        /* return error to stop iteration */
+        return AWS_COMMON_HASH_TABLE_ITER_ERROR;
+    }
+    if (!was_created) {
+        /* no new entry created, clean up the copy ourselves */
+        aws_string_destroy(key_copy);
+    }
+    return AWS_COMMON_HASH_TABLE_ITER_CONTINUE;
+}
+
+int aws_http_alpn_map_init_copy(
+    struct aws_allocator *allocator,
+    struct aws_hash_table *dest,
+    struct aws_hash_table *src) {
+    if (aws_http_alpn_map_init(allocator, dest)) {
+        return AWS_OP_ERR;
+    }
+    struct s_copy_alpn_string_map_context context;
+    context.allocator = allocator;
+    context.map = dest;
+    /* make a deep copy of the map */
+    if (aws_hash_table_foreach(src, s_copy_alpn_string_map, &context)) {
+        int error_code = aws_last_error();
+        AWS_LOGF_ERROR(
+            AWS_LS_HTTP_CONNECTION,
+            "Failed to copy ALPN map with error code %d (%s)",
+            error_code,
+            aws_error_name(error_code));
+        aws_hash_table_clean_up(dest);
+        return AWS_OP_ERR;
+    }
+    return AWS_OP_SUCCESS;
+}
+
 int aws_http_client_connect_internal(
     const struct aws_http_client_connection_options *orig_options,
     aws_http_proxy_request_transform_fn *proxy_request_transform) {
@@ -884,20 +1008,25 @@ int aws_http_client_connect_internal(
         AWS_LOGF_ERROR(AWS_LS_HTTP_CONNECTION, "static: http connection options are null.");
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
-
     struct aws_http_client_bootstrap *http_bootstrap = NULL;
     struct aws_string *host_name = NULL;
     int err = 0;
 
     /* make copy of options, and add defaults for missing optional structs */
     struct aws_http_client_connection_options options = *orig_options;
+    if (options.prior_knowledge_http2 && options.tls_options) {
+        AWS_LOGF_ERROR(AWS_LS_HTTP_CONNECTION, "static: HTTP/2 prior knowledge only works with cleartext TCP.");
+        return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+    }
 
-    const struct aws_http1_connection_options default_http1_options = AWS_HTTP1_CONNECTION_OPTIONS_INIT;
+    struct aws_http1_connection_options default_http1_options;
+    AWS_ZERO_STRUCT(default_http1_options);
     if (options.http1_options == NULL) {
         options.http1_options = &default_http1_options;
     }
 
-    const struct aws_http2_connection_options default_http2_options = AWS_HTTP2_CONNECTION_OPTIONS_INIT;
+    struct aws_http2_connection_options default_http2_options;
+    AWS_ZERO_STRUCT(default_http2_options);
     if (options.http2_options == NULL) {
         options.http2_options = &default_http2_options;
     }
@@ -916,13 +1045,16 @@ int aws_http_client_connect_internal(
     }
 
     struct aws_http2_setting *setting_array = NULL;
+    struct aws_hash_table *alpn_string_map = NULL;
     if (!aws_mem_acquire_many(
             options.allocator,
-            2,
+            3,
             &http_bootstrap,
             sizeof(struct aws_http_client_bootstrap),
             &setting_array,
-            options.http2_options->num_initial_settings * sizeof(struct aws_http2_setting))) {
+            options.http2_options->num_initial_settings * sizeof(struct aws_http2_setting),
+            &alpn_string_map,
+            sizeof(struct aws_hash_table))) {
         goto error;
     }
 
@@ -931,6 +1063,7 @@ int aws_http_client_connect_internal(
     http_bootstrap->alloc = options.allocator;
     http_bootstrap->is_using_tls = options.tls_options != NULL;
     http_bootstrap->manual_window_management = options.manual_window_management;
+    http_bootstrap->prior_knowledge_http2 = options.prior_knowledge_http2;
     http_bootstrap->initial_window_size = options.initial_window_size;
     http_bootstrap->user_data = options.user_data;
     http_bootstrap->on_setup = options.on_setup;
@@ -948,6 +1081,13 @@ int aws_http_client_connect_internal(
         http_bootstrap->http2_options.initial_settings_array = setting_array;
     }
 
+    if (options.alpn_string_map) {
+        if (aws_http_alpn_map_init_copy(options.allocator, alpn_string_map, options.alpn_string_map)) {
+            goto error;
+        }
+        http_bootstrap->alpn_string_map = alpn_string_map;
+    }
+
     if (options.monitoring_options) {
         http_bootstrap->monitoring_options = *options.monitoring_options;
     }
@@ -987,7 +1127,7 @@ int aws_http_client_connect_internal(
 
 error:
     if (http_bootstrap) {
-        aws_mem_release(http_bootstrap->alloc, http_bootstrap);
+        aws_http_client_bootstrap_destroy(http_bootstrap);
     }
 
     if (host_name) {
@@ -999,11 +1139,20 @@ error:
 
 int aws_http_client_connect(const struct aws_http_client_connection_options *options) {
     aws_http_fatal_assert_library_initialized();
+    if (options->prior_knowledge_http2 && options->tls_options) {
+        AWS_LOGF_ERROR(AWS_LS_HTTP_CONNECTION, "static: HTTP/2 prior knowledge only works with cleartext TCP.");
+        return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+    }
 
     if (options->proxy_options != NULL) {
         return aws_http_client_connect_via_proxy(options);
     } else {
-        return aws_http_client_connect_internal(options, NULL);
+        if (!options->proxy_ev_settings || options->proxy_ev_settings->env_var_type != AWS_HPEV_ENABLE) {
+            return aws_http_client_connect_internal(options, NULL);
+        } else {
+            /* Proxy through envrionment variable is enabled */
+            return aws_http_client_connect_via_proxy(options);
+        }
     }
 }
 
@@ -1055,6 +1204,5 @@ uint32_t aws_http_connection_get_next_stream_id(struct aws_http_connection *conn
     } else {
         connection->next_stream_id += 2;
     }
-
     return next_id;
 }
