@@ -276,20 +276,23 @@ static int s_packet_handler_suback(
         (void *)connection,
         suback.packet_identifier);
 
-    struct aws_hash_element *elem = NULL;
+    struct aws_mqtt_request *request = NULL;
 
     { /* BEGIN CRITICAL SECTION */
         mqtt_connection_lock_synced_data(connection);
+        struct aws_hash_element *elem = NULL;
         aws_hash_table_find(&connection->synced_data.outstanding_requests_table, &suback.packet_identifier, &elem);
+        if (elem != NULL) {
+            request = elem->value;
+        }
         mqtt_connection_unlock_synced_data(connection);
     } /* END CRITICAL SECTION */
 
-    if (elem == NULL) {
+    if (request == NULL) {
         /* no corresponding request found */
         goto done;
     }
 
-    struct aws_mqtt_request *request = elem->value;
     struct subscribe_task_arg *task_arg = request->on_complete_ud;
     size_t request_topics_len = aws_array_list_length(&task_arg->topics);
     size_t suback_return_code_len = aws_array_list_length(&suback.return_codes);
@@ -819,6 +822,44 @@ uint16_t mqtt_create_request(
             aws_raise_error(AWS_ERROR_MQTT_NOT_CONNECTED);
             return 0;
         }
+        /**
+         * Find a free packet ID.
+         * QoS 0 PUBLISH packets don't actually need an ID on the wire,
+         * but we assign them internally anyway just so everything has a unique ID.
+         *
+         * Yes, this is an O(N) search.
+         * We remember the last ID we assigned, so it's O(1) in the common case.
+         * But it's theoretically possible to reach O(N) where N is just above 64000
+         * if the user is letting a ton of un-ack'd messages queue up
+         */
+        uint16_t search_start = connection->synced_data.packet_id;
+        struct aws_hash_element *elem = NULL;
+        while (true) {
+            /* Increment ID, watch out for overflow, ID cannot be 0 */
+            if (connection->synced_data.packet_id == UINT16_MAX) {
+                connection->synced_data.packet_id = 1;
+            } else {
+                connection->synced_data.packet_id++;
+            }
+
+            /* Is there already an outstanding request using this ID? */
+            aws_hash_table_find(
+                &connection->synced_data.outstanding_requests_table, &connection->synced_data.packet_id, &elem);
+
+            if (elem == NULL) {
+                /* Found a free ID! Break out of loop */
+                break;
+            } else if (connection->synced_data.packet_id == search_start) {
+                /* Every ID is taken */
+                mqtt_connection_unlock_synced_data(connection);
+                AWS_LOGF_ERROR(
+                    AWS_LS_MQTT_CLIENT,
+                    "id=%p: Queue is full. No more packet IDs are available at this time.",
+                    (void *)connection);
+                aws_raise_error(AWS_ERROR_MQTT_QUEUE_FULL);
+                return 0;
+            }
+        }
 
         next_request = aws_memory_pool_acquire(&connection->synced_data.requests_pool);
         if (!next_request) {
@@ -826,20 +867,8 @@ uint16_t mqtt_create_request(
             return 0;
         }
         memset(next_request, 0, sizeof(struct aws_mqtt_request));
-        struct aws_hash_element *elem = NULL;
-        uint16_t next_id = 0;
-        /**
-         * QoS 0 publish and PINGREQ don't need a packet ID, but, we may still need a ID for publish QoS 0,
-         * For user to keep all data alive, an ID will help.
-         */
-        do {
-            /* find the next unused packet id for the new request */
-            ++next_id;
-            aws_hash_table_find(&connection->synced_data.outstanding_requests_table, &next_id, &elem);
 
-        } while (elem);
-        AWS_ASSERT(next_id); /* Somehow have UINT16_MAX outstanding requests, definitely a bug */
-        next_request->packet_id = next_id;
+        next_request->packet_id = connection->synced_data.packet_id;
 
         if (aws_hash_table_put(
                 &connection->synced_data.outstanding_requests_table, &next_request->packet_id, next_request, NULL)) {
@@ -887,21 +916,38 @@ uint16_t mqtt_create_request(
 
 void mqtt_request_complete(struct aws_mqtt_client_connection *connection, int error_code, uint16_t packet_id) {
 
-    struct aws_hash_element *elem = NULL;
-
     AWS_LOGF_TRACE(
         AWS_LS_MQTT_CLIENT,
         "id=%p: message id %" PRIu16 " completed with error code %d, removing from outstanding requests list.",
         (void *)connection,
         packet_id,
         error_code);
+
+    bool found_request = false;
+    aws_mqtt_op_complete_fn *on_complete = NULL;
+    void *on_complete_ud = NULL;
+
     { /* BEGIN CRITICAL SECTION */
         mqtt_connection_lock_synced_data(connection);
+        struct aws_hash_element *elem = NULL;
         aws_hash_table_find(&connection->synced_data.outstanding_requests_table, &packet_id, &elem);
+        if (elem != NULL) {
+            found_request = true;
+
+            struct aws_mqtt_request *request = elem->value;
+            on_complete = request->on_complete;
+            on_complete_ud = request->on_complete_ud;
+
+            /* clean up request resources */
+            aws_hash_table_remove_element(&connection->synced_data.outstanding_requests_table, elem);
+            /* remove the request from the list, which is thread_data.ongoing_requests_list */
+            aws_linked_list_remove(&request->list_node);
+            aws_memory_pool_release(&connection->synced_data.requests_pool, request);
+        }
         mqtt_connection_unlock_synced_data(connection);
     } /* END CRITICAL SECTION */
 
-    if (elem == NULL) {
+    if (!found_request) {
         AWS_LOGF_DEBUG(
             AWS_LS_MQTT_CLIENT,
             "id=%p: received completion for message id %" PRIu16
@@ -912,19 +958,10 @@ void mqtt_request_complete(struct aws_mqtt_client_connection *connection, int er
         return;
     }
 
-    struct aws_mqtt_request *request = elem->value;
     /* Invoke the complete callback. */
-    if (request->on_complete) {
-        request->on_complete(connection, request->packet_id, error_code, request->on_complete_ud);
+    if (on_complete) {
+        on_complete(connection, packet_id, error_code, on_complete_ud);
     }
-    /* remove the request from the list, which is the outgoing request list, and clean up the resource */
-    aws_linked_list_remove(&request->list_node);
-    { /* BEGIN CRITICAL SECTION */
-        mqtt_connection_lock_synced_data(connection);
-        aws_hash_table_remove_element(&connection->synced_data.outstanding_requests_table, elem);
-        aws_memory_pool_release(&connection->synced_data.requests_pool, request);
-        mqtt_connection_unlock_synced_data(connection);
-    } /* END CRITICAL SECTION */
 }
 
 struct mqtt_shutdown_task {
