@@ -167,9 +167,9 @@ union chacha20_poly1305_seal_data {
 #if defined(OPENSSL_X86_64) && !defined(OPENSSL_NO_ASM)
 
 OPENSSL_STATIC_ASSERT(sizeof(union chacha20_poly1305_open_data) == 48,
-                      _wrong_chacha20_poly1305_open_data_size);
+                      _wrong_chacha20_poly1305_open_data_size)
 OPENSSL_STATIC_ASSERT(sizeof(union chacha20_poly1305_seal_data) == 48 + 8 + 8,
-                      _wrong_chacha20_poly1305_seal_data_size);
+                      _wrong_chacha20_poly1305_seal_data_size)
 
 OPENSSL_INLINE int chacha20_poly1305_asm_capable(void) {
   const int sse41_capable = (OPENSSL_ia32cap_P[1] & (1 << 19)) != 0;
