@@ -25,10 +25,10 @@
 
 
 OPENSSL_STATIC_ASSERT(sizeof(CRYPTO_MUTEX) >= sizeof(pthread_rwlock_t),
-                      CRYPTO_MUTEX_is_too_small);
+                      CRYPTO_MUTEX_is_too_small)
 #if defined(__GNUC__) || defined(__clang__)
 OPENSSL_STATIC_ASSERT(alignof(CRYPTO_MUTEX) >= alignof(pthread_rwlock_t),
-                      CRYPTO_MUTEX_has_insufficient_alignment);
+                      CRYPTO_MUTEX_has_insufficient_alignment)
 #endif
 
 void CRYPTO_MUTEX_init(CRYPTO_MUTEX *lock) {
