@@ -21,8 +21,8 @@
 #include "crypto/s2n_fips.h"
 
 #include "error/s2n_errno.h"
-
 #include "crypto/s2n_hash.h"
+#include "crypto/s2n_rsa_signing.h"
 
 #include "tls/extensions/s2n_extension_list.h"
 #include "tls/extensions/s2n_server_key_share.h"
@@ -70,9 +70,8 @@ static S2N_RESULT s2n_generate_client_session_id(struct s2n_connection *conn)
         return S2N_RESULT_OK;
     }
 
-    /* Generate the session id for TLS1.3 if in middlebox compatibility mode.
-     * For now, we default to middlebox compatibility mode unless using QUIC. */
-    if (conn->config->quic_enabled) {
+    /* Only generate the session id for TLS1.3 if in middlebox compatibility mode */
+    if (conn->client_protocol_version >= S2N_TLS13 && !s2n_is_middlebox_compat_enabled(conn)) {
         return S2N_RESULT_OK;
     }
 
@@ -219,8 +218,8 @@ static int s2n_parse_client_hello(struct s2n_connection *conn)
 
     POSIX_GUARD(s2n_stuffer_read_uint8(in, &conn->session_id_len));
     S2N_ERROR_IF(conn->session_id_len > S2N_TLS_SESSION_ID_MAX_LEN || conn->session_id_len > s2n_stuffer_data_available(in), S2N_ERR_BAD_MESSAGE);
-
-    POSIX_GUARD(s2n_stuffer_read_bytes(in, conn->session_id, conn->session_id_len));
+    POSIX_GUARD(s2n_blob_init(&client_hello->session_id, s2n_stuffer_raw_read(in, conn->session_id_len), conn->session_id_len));
+    POSIX_CHECKED_MEMCPY(conn->session_id, client_hello->session_id.data, conn->session_id_len);
 
     uint16_t cipher_suites_length = 0;
     POSIX_GUARD(s2n_stuffer_read_uint16(in, &cipher_suites_length));
@@ -257,8 +256,45 @@ static int s2n_parse_client_hello(struct s2n_connection *conn)
     return S2N_SUCCESS;
 }
 
+bool s2n_is_tls_12_self_downgrade_required(struct s2n_connection *conn) {
+    if ((conn->mode == S2N_SERVER)
+          && (conn->config != NULL)
+          && conn->config->is_rsa_cert_configured
+          && !s2n_is_tls13_fully_supported()) {
+        /* RSA PSS is required for TLS 1.3 connections if RSA is used. So if we are a server that doesn't support RSA PSS,
+         * and there's a possibility that an RSA Certificate could be picked by a client connection, then self-downgrade
+         * connection to TLS 1.2. */
+        return true;
+    }
+
+    s2n_cert_auth_type client_auth_status = S2N_CERT_AUTH_NONE;
+    s2n_connection_get_client_auth_type(conn, &client_auth_status);
+
+    if ((conn->mode == S2N_SERVER)
+        && (client_auth_status != S2N_CERT_AUTH_NONE)
+        && !s2n_is_tls13_fully_supported()) {
+        /* If we're a Server that is going to request a client certificate, and we don't support RSA PSS, then
+         * downgrade to TLS 1.2 in case the client attempts to send an RSA PSS Certificate. */
+       return true;
+    }
+
+    if ((conn->mode == S2N_CLIENT) && !s2n_is_tls13_fully_supported()) {
+        /* There are some TLS Servers in the wild that will always choose RSA PSS if the client claims to support TLS 1.3
+         * even if the client does not advertise support for RSA PSS in the SignatureScheme extension. In order to work
+         * around this issue, our client should self-downgrade to TLS 1.2 so that a successful TLS 1.2 connection can be
+         * made instead of rejecting a TLS 1.3 ServerHello with RSA PSS due to it containing algorithms that we don't
+         * support. */
+        return true;
+    }
+
+  return false;
+
+}
+
 int s2n_process_client_hello(struct s2n_connection *conn)
 {
+    POSIX_ENSURE_REF(conn);
+
     /* Client hello is parsed and config is finalized.
      * Negotiate protocol version, cipher suite, ALPN, select a cert, etc. */
     struct s2n_client_hello *client_hello = &conn->client_hello;
@@ -266,8 +302,7 @@ int s2n_process_client_hello(struct s2n_connection *conn)
     const struct s2n_security_policy *security_policy;
     POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
 
-    /* Ensure that highest supported version is set correctly */
-    if (!s2n_security_policy_supports_tls13(security_policy)) {
+    if (s2n_is_tls_12_self_downgrade_required(conn) || !s2n_security_policy_supports_tls13(security_policy)) {
         conn->server_protocol_version = MIN(conn->server_protocol_version, S2N_TLS12);
         conn->actual_protocol_version = MIN(conn->server_protocol_version, S2N_TLS12);
     }
@@ -293,7 +328,7 @@ int s2n_process_client_hello(struct s2n_connection *conn)
         POSIX_BAIL(S2N_ERR_PROTOCOL_VERSION_UNSUPPORTED);
     }
 
-    if (conn->config->quic_enabled) {
+    if (s2n_connection_is_quic_enabled(conn)) {
         POSIX_ENSURE(conn->actual_protocol_version >= S2N_TLS13, S2N_ERR_PROTOCOL_VERSION_UNSUPPORTED);
     }
 
@@ -378,15 +413,15 @@ int s2n_client_hello_recv(struct s2n_connection *conn)
 
 int s2n_client_hello_send(struct s2n_connection *conn)
 {
+    POSIX_ENSURE_REF(conn);
+
     const struct s2n_security_policy *security_policy;
     POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
 
     const struct s2n_cipher_preferences *cipher_preferences = security_policy->cipher_preferences;
     POSIX_ENSURE_REF(cipher_preferences);
 
-    /* Check whether cipher preference supports TLS 1.3. If it doesn't,
-       our highest supported version is S2N_TLS12 */
-    if (!s2n_security_policy_supports_tls13(security_policy)) {
+    if (s2n_is_tls_12_self_downgrade_required(conn) || !s2n_security_policy_supports_tls13(security_policy)) {
         conn->client_protocol_version = MIN(conn->client_protocol_version, S2N_TLS12);
         conn->actual_protocol_version = MIN(conn->actual_protocol_version, S2N_TLS12);
     }
@@ -497,15 +532,14 @@ int s2n_sslv2_client_hello_recv(struct s2n_connection *conn)
     POSIX_GUARD(s2n_conn_find_name_matching_certs(conn));
 
     POSIX_GUARD(s2n_set_cipher_as_sslv2_server(conn, client_hello->cipher_suites.data, client_hello->cipher_suites.size / S2N_SSLv2_CIPHER_SUITE_LEN));
-    POSIX_GUARD(s2n_choose_default_sig_scheme(conn, &conn->handshake_params.conn_sig_scheme));
+    POSIX_GUARD(s2n_choose_default_sig_scheme(conn, &conn->handshake_params.conn_sig_scheme, S2N_SERVER));
     POSIX_GUARD(s2n_select_certs_for_server_auth(conn, &conn->handshake_params.our_chain_and_key));
 
     S2N_ERROR_IF(session_id_length > s2n_stuffer_data_available(in), S2N_ERR_BAD_MESSAGE);
+    POSIX_GUARD(s2n_blob_init(&client_hello->session_id, s2n_stuffer_raw_read(in, session_id_length), session_id_length));
     if (session_id_length > 0 && session_id_length <= S2N_TLS_SESSION_ID_MAX_LEN) {
-        POSIX_GUARD(s2n_stuffer_read_bytes(in, conn->session_id, session_id_length));
+        POSIX_CHECKED_MEMCPY(conn->session_id, client_hello->session_id.data, session_id_length);
         conn->session_id_len = (uint8_t) session_id_length;
-    } else {
-        POSIX_GUARD(s2n_stuffer_skip_read(in, session_id_length));
     }
 
     struct s2n_blob b = {0};
@@ -562,3 +596,24 @@ ssize_t s2n_client_hello_get_extension_by_id(struct s2n_client_hello *ch, s2n_tl
     POSIX_CHECKED_MEMCPY(out, parsed_extension->extension.data, len);
     return len;
 }
+
+int s2n_client_hello_get_session_id_length(struct s2n_client_hello *ch, uint32_t *out_length)
+{
+    POSIX_ENSURE_REF(ch);
+    POSIX_ENSURE_REF(out_length);
+    *out_length = ch->session_id.size;
+    return S2N_SUCCESS;
+}
+
+int s2n_client_hello_get_session_id(struct s2n_client_hello *ch, uint8_t *out, uint32_t *out_length, uint32_t max_length)
+{
+    POSIX_ENSURE_REF(ch);
+    POSIX_ENSURE_REF(out);
+    POSIX_ENSURE_REF(out_length);
+
+    uint32_t len = min_size(&ch->session_id, max_length);
+    POSIX_CHECKED_MEMCPY(out, ch->session_id.data, len);
+    *out_length = len;
+
+    return S2N_SUCCESS;
+}
