@@ -45,6 +45,8 @@
 
 #define S2N_TLS_PROTOCOL_VERSION_LEN    2
 
+#define S2N_PEER_MODE(our_mode) ((our_mode + 1) % 2)
+
 #define is_handshake_complete(conn) (APPLICATION_DATA == s2n_conn_get_current_message_type(conn))
 
 typedef enum {
@@ -75,12 +77,8 @@ struct s2n_connection {
     void *send_io_context;
     void *recv_io_context;
 
-    /* Has the user set their own I/O callbacks or is this connection using the
-     * default socket-based I/O set by s2n */
-    uint8_t managed_io;
-
     /* Is this connection using CORK/SO_RCVLOWAT optimizations? Only valid when the connection is using
-     * managed_io
+     * managed_send_io
      */
     unsigned corked_io:1;
 
@@ -174,7 +172,7 @@ struct s2n_connection {
     struct s2n_psk_parameters psk_params;
 
     /* The PRF needs some storage elements to work with */
-    struct s2n_prf_working_space prf_space;
+    struct s2n_prf_working_space *prf_space;
 
     /* Whether to use client_cert_auth_type stored in s2n_config or in this s2n_connection.
      *
@@ -324,6 +322,11 @@ struct s2n_connection {
     /* Cookie extension data */
     struct s2n_stuffer cookie_stuffer;
 
+    /* Has the user set their own I/O callbacks or is this connection using the
+     * default socket-based I/O set by s2n */
+    unsigned managed_send_io:1;
+    unsigned managed_recv_io:1;
+
     /* Key update data */
     unsigned key_update_pending:1;
 
@@ -344,6 +347,12 @@ struct s2n_connection {
     /* Have we received a close notify alert from the peer. */
     unsigned close_notify_received:1;
 
+    /* Connection negotiated an EMS */
+    unsigned ems_negotiated:1;
+
+    /* Connection can be used by a QUIC implementation */
+    unsigned quic_enabled:1;
+
     /* Flags to prevent users from calling methods recursively.
      * This can be an easy mistake to make when implementing send/receive callbacks.
      */
@@ -357,9 +366,6 @@ struct s2n_connection {
     uint32_t server_max_early_data_size;
     struct s2n_blob server_early_data_context;
     uint32_t server_keying_material_lifetime;
-
-    /* To avoid allocating memory for hash objects, we reuse one temporary hash object. */
-    struct s2n_hash_state hash_workspace;
 };
 
 int s2n_connection_is_managed_corked(const struct s2n_connection *s2n_connection);
