@@ -23,6 +23,7 @@
 #include "tls/s2n_cipher_suites.h"
 #include "tls/s2n_connection.h"
 #include "tls/s2n_prf.h"
+#include "tls/s2n_tls.h"
 
 #include "stuffer/s2n_stuffer.h"
 
@@ -35,19 +36,24 @@
 #include "utils/s2n_blob.h"
 #include "utils/s2n_mem.h"
 
-static int s2n_sslv3_prf(struct s2n_prf_working_space *ws, struct s2n_blob *secret, struct s2n_blob *seed_a,
+static int s2n_sslv3_prf(struct s2n_connection *conn, struct s2n_blob *secret, struct s2n_blob *seed_a,
         struct s2n_blob *seed_b, struct s2n_blob *seed_c, struct s2n_blob *out)
 {
-    struct s2n_hash_state *md5 = &ws->ssl3.md5;
-    struct s2n_hash_state *sha1 = &ws->ssl3.sha1;
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(conn->handshake.hashes);
+    struct s2n_hash_state *workspace = &conn->handshake.hashes->hash_workspace;
 
     uint32_t outputlen = out->size;
     uint8_t *output = out->data;
     uint8_t iteration = 1;
 
+    uint8_t md5_digest[MD5_DIGEST_LENGTH] = { 0 }, sha_digest[SHA_DIGEST_LENGTH] = { 0 };
+
     uint8_t A = 'A';
     while (outputlen) {
+        struct s2n_hash_state *sha1 = workspace;
         POSIX_GUARD(s2n_hash_reset(sha1));
+        POSIX_GUARD(s2n_hash_init(sha1, S2N_HASH_SHA1));
 
         for (int i = 0; i < iteration; i++) {
             POSIX_GUARD(s2n_hash_update(sha1, &A, 1));
@@ -63,16 +69,18 @@ static int s2n_sslv3_prf(struct s2n_prf_working_space *ws, struct s2n_blob *secr
             }
         }
 
-        POSIX_GUARD(s2n_hash_digest(sha1, ws->ssl3.sha1_digest, sizeof(ws->ssl3.sha1_digest)));
+        POSIX_GUARD(s2n_hash_digest(sha1, sha_digest, sizeof(sha_digest)));
 
+        struct s2n_hash_state *md5 = workspace;
         POSIX_GUARD(s2n_hash_reset(md5));
+        POSIX_GUARD(s2n_hash_init(md5, S2N_HASH_MD5));
         POSIX_GUARD(s2n_hash_update(md5, secret->data, secret->size));
-        POSIX_GUARD(s2n_hash_update(md5, ws->ssl3.sha1_digest, sizeof(ws->ssl3.sha1_digest)));
-        POSIX_GUARD(s2n_hash_digest(md5, ws->ssl3.md5_digest, sizeof(ws->ssl3.md5_digest)));
+        POSIX_GUARD(s2n_hash_update(md5, sha_digest, sizeof(sha_digest)));
+        POSIX_GUARD(s2n_hash_digest(md5, md5_digest, sizeof(md5_digest)));
 
-        uint32_t bytes_to_copy = MIN(outputlen, sizeof(ws->ssl3.md5_digest));
+        uint32_t bytes_to_copy = MIN(outputlen, sizeof(md5_digest));
 
-        POSIX_CHECKED_MEMCPY(output, ws->ssl3.md5_digest, bytes_to_copy);
+        POSIX_CHECKED_MEMCPY(output, md5_digest, bytes_to_copy);
 
         outputlen -= bytes_to_copy;
         output += bytes_to_copy;
@@ -82,31 +90,28 @@ static int s2n_sslv3_prf(struct s2n_prf_working_space *ws, struct s2n_blob *secr
         iteration++;
     }
 
-    POSIX_GUARD(s2n_hash_reset(md5));
-    POSIX_GUARD(s2n_hash_reset(sha1));
-
     return 0;
 }
 
 #if !defined(OPENSSL_IS_BORINGSSL) && !defined(OPENSSL_IS_AWSLC)
 static int s2n_evp_hmac_p_hash_new(struct s2n_prf_working_space *ws)
 {
-    POSIX_ENSURE_REF(ws->tls.p_hash.evp_hmac.evp_digest.ctx = S2N_EVP_MD_CTX_NEW());
+    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.evp_digest.ctx = S2N_EVP_MD_CTX_NEW());
     return 0;
 }
 
 static int s2n_evp_hmac_p_hash_digest_init(struct s2n_prf_working_space *ws)
 {
-    POSIX_ENSURE_REF(ws->tls.p_hash.evp_hmac.evp_digest.md);
-    POSIX_ENSURE_REF(ws->tls.p_hash.evp_hmac.evp_digest.ctx);
-    POSIX_ENSURE_REF(ws->tls.p_hash.evp_hmac.mac_key);
+    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.evp_digest.md);
+    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.evp_digest.ctx);
+    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.mac_key);
  
     /* Ignore the MD5 check when in FIPS mode to comply with the TLS 1.0 RFC */
     if (s2n_is_in_fips_mode()) {
-        POSIX_GUARD(s2n_digest_allow_md5_for_fips(&ws->tls.p_hash.evp_hmac.evp_digest));
+        POSIX_GUARD(s2n_digest_allow_md5_for_fips(&ws->p_hash.evp_hmac.evp_digest));
     }
 
-    POSIX_GUARD_OSSL(EVP_DigestSignInit(ws->tls.p_hash.evp_hmac.evp_digest.ctx, NULL, ws->tls.p_hash.evp_hmac.evp_digest.md, NULL, ws->tls.p_hash.evp_hmac.mac_key),
+    POSIX_GUARD_OSSL(EVP_DigestSignInit(ws->p_hash.evp_hmac.evp_digest.ctx, NULL, ws->p_hash.evp_hmac.evp_digest.md, NULL, ws->p_hash.evp_hmac.mac_key),
            S2N_ERR_P_HASH_INIT_FAILED);
 
     return 0;
@@ -118,30 +123,30 @@ static int s2n_evp_hmac_p_hash_init(struct s2n_prf_working_space *ws, s2n_hmac_a
     switch (alg) {
     case S2N_HMAC_SSLv3_MD5:
     case S2N_HMAC_MD5:
-        ws->tls.p_hash.evp_hmac.evp_digest.md = EVP_md5();
+        ws->p_hash.evp_hmac.evp_digest.md = EVP_md5();
         break;
     case S2N_HMAC_SSLv3_SHA1:
     case S2N_HMAC_SHA1:
-        ws->tls.p_hash.evp_hmac.evp_digest.md = EVP_sha1();
+        ws->p_hash.evp_hmac.evp_digest.md = EVP_sha1();
         break;
     case S2N_HMAC_SHA224:
-        ws->tls.p_hash.evp_hmac.evp_digest.md = EVP_sha224();
+        ws->p_hash.evp_hmac.evp_digest.md = EVP_sha224();
         break;
     case S2N_HMAC_SHA256:
-        ws->tls.p_hash.evp_hmac.evp_digest.md = EVP_sha256();
+        ws->p_hash.evp_hmac.evp_digest.md = EVP_sha256();
         break;
     case S2N_HMAC_SHA384:
-        ws->tls.p_hash.evp_hmac.evp_digest.md = EVP_sha384();
+        ws->p_hash.evp_hmac.evp_digest.md = EVP_sha384();
         break;
     case S2N_HMAC_SHA512:
-        ws->tls.p_hash.evp_hmac.evp_digest.md = EVP_sha512();
+        ws->p_hash.evp_hmac.evp_digest.md = EVP_sha512();
         break;
     default:
         POSIX_BAIL(S2N_ERR_P_HASH_INVALID_ALGORITHM);
     }
 
     /* Initialize the mac key using the provided secret */
-    POSIX_ENSURE_REF(ws->tls.p_hash.evp_hmac.mac_key = EVP_PKEY_new_mac_key(EVP_PKEY_HMAC, NULL, secret->data, secret->size));
+    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.mac_key = EVP_PKEY_new_mac_key(EVP_PKEY_HMAC, NULL, secret->data, secret->size));
 
     /* Initialize the message digest context with the above message digest and mac key */
     return s2n_evp_hmac_p_hash_digest_init(ws);
@@ -149,7 +154,7 @@ static int s2n_evp_hmac_p_hash_init(struct s2n_prf_working_space *ws, s2n_hmac_a
 
 static int s2n_evp_hmac_p_hash_update(struct s2n_prf_working_space *ws, const void *data, uint32_t size)
 {
-    POSIX_GUARD_OSSL(EVP_DigestSignUpdate(ws->tls.p_hash.evp_hmac.evp_digest.ctx, data, (size_t)size), S2N_ERR_P_HASH_UPDATE_FAILED);
+    POSIX_GUARD_OSSL(EVP_DigestSignUpdate(ws->p_hash.evp_hmac.evp_digest.ctx, data, (size_t)size), S2N_ERR_P_HASH_UPDATE_FAILED);
 
     return 0;
 }
@@ -159,14 +164,14 @@ static int s2n_evp_hmac_p_hash_digest(struct s2n_prf_working_space *ws, void *di
     /* EVP_DigestSign API's require size_t data structures */
     size_t digest_size = size;
 
-    POSIX_GUARD_OSSL(EVP_DigestSignFinal(ws->tls.p_hash.evp_hmac.evp_digest.ctx, (unsigned char *)digest, &digest_size), S2N_ERR_P_HASH_FINAL_FAILED);
+    POSIX_GUARD_OSSL(EVP_DigestSignFinal(ws->p_hash.evp_hmac.evp_digest.ctx, (unsigned char *)digest, &digest_size), S2N_ERR_P_HASH_FINAL_FAILED);
 
     return 0;
 }
 
 static int s2n_evp_hmac_p_hash_wipe(struct s2n_prf_working_space *ws)
 {
-  POSIX_GUARD_OSSL(S2N_EVP_MD_CTX_RESET(ws->tls.p_hash.evp_hmac.evp_digest.ctx), S2N_ERR_P_HASH_WIPE_FAILED);
+  POSIX_GUARD_OSSL(S2N_EVP_MD_CTX_RESET(ws->p_hash.evp_hmac.evp_digest.ctx), S2N_ERR_P_HASH_WIPE_FAILED);
 
     return 0;
 }
@@ -184,18 +189,18 @@ static int s2n_evp_hmac_p_hash_cleanup(struct s2n_prf_working_space *ws)
     POSIX_GUARD(s2n_evp_hmac_p_hash_wipe(ws));
 
     /* Free mac key - PKEYs cannot be reused */
-    POSIX_ENSURE_REF(ws->tls.p_hash.evp_hmac.mac_key);
-    EVP_PKEY_free(ws->tls.p_hash.evp_hmac.mac_key);
-    ws->tls.p_hash.evp_hmac.mac_key = NULL;
+    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.mac_key);
+    EVP_PKEY_free(ws->p_hash.evp_hmac.mac_key);
+    ws->p_hash.evp_hmac.mac_key = NULL;
 
     return 0;
 }
 
 static int s2n_evp_hmac_p_hash_free(struct s2n_prf_working_space *ws)
 {
-    POSIX_ENSURE_REF(ws->tls.p_hash.evp_hmac.evp_digest.ctx);
-    S2N_EVP_MD_CTX_FREE(ws->tls.p_hash.evp_hmac.evp_digest.ctx);
-    ws->tls.p_hash.evp_hmac.evp_digest.ctx = NULL;
+    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.evp_digest.ctx);
+    S2N_EVP_MD_CTX_FREE(ws->p_hash.evp_hmac.evp_digest.ctx);
+    ws->p_hash.evp_hmac.evp_digest.ctx = NULL;
 
     return 0;
 }
@@ -213,29 +218,29 @@ static const struct s2n_p_hash_hmac s2n_evp_hmac = {
 
 static int s2n_hmac_p_hash_new(struct s2n_prf_working_space *ws)
 {
-    POSIX_GUARD(s2n_hmac_new(&ws->tls.p_hash.s2n_hmac));
+    POSIX_GUARD(s2n_hmac_new(&ws->p_hash.s2n_hmac));
 
-    return s2n_hmac_init(&ws->tls.p_hash.s2n_hmac, S2N_HMAC_NONE, NULL, 0);
+    return s2n_hmac_init(&ws->p_hash.s2n_hmac, S2N_HMAC_NONE, NULL, 0);
 }
 
 static int s2n_hmac_p_hash_init(struct s2n_prf_working_space *ws, s2n_hmac_algorithm alg, struct s2n_blob *secret)
 {
-    return s2n_hmac_init(&ws->tls.p_hash.s2n_hmac, alg, secret->data, secret->size);
+    return s2n_hmac_init(&ws->p_hash.s2n_hmac, alg, secret->data, secret->size);
 }
 
 static int s2n_hmac_p_hash_update(struct s2n_prf_working_space *ws, const void *data, uint32_t size)
 {
-    return s2n_hmac_update(&ws->tls.p_hash.s2n_hmac, data, size);
+    return s2n_hmac_update(&ws->p_hash.s2n_hmac, data, size);
 }
 
 static int s2n_hmac_p_hash_digest(struct s2n_prf_working_space *ws, void *digest, uint32_t size)
 {
-    return s2n_hmac_digest(&ws->tls.p_hash.s2n_hmac, digest, size);
+    return s2n_hmac_digest(&ws->p_hash.s2n_hmac, digest, size);
 }
 
 static int s2n_hmac_p_hash_reset(struct s2n_prf_working_space *ws)
 {
-    return s2n_hmac_reset(&ws->tls.p_hash.s2n_hmac);
+    return s2n_hmac_reset(&ws->p_hash.s2n_hmac);
 }
 
 static int s2n_hmac_p_hash_cleanup(struct s2n_prf_working_space *ws)
@@ -245,7 +250,7 @@ static int s2n_hmac_p_hash_cleanup(struct s2n_prf_working_space *ws)
 
 static int s2n_hmac_p_hash_free(struct s2n_prf_working_space *ws)
 {
-    return s2n_hmac_free(&ws->tls.p_hash.s2n_hmac);
+    return s2n_hmac_free(&ws->p_hash.s2n_hmac);
 }
 
 static const struct s2n_p_hash_hmac s2n_hmac = {
@@ -264,7 +269,7 @@ static int s2n_p_hash(struct s2n_prf_working_space *ws, s2n_hmac_algorithm alg,
     uint8_t digest_size;
     POSIX_GUARD(s2n_hmac_digest_size(alg, &digest_size));
 
-    const struct s2n_p_hash_hmac *hmac = ws->tls.p_hash_hmac_impl;
+    const struct s2n_p_hash_hmac *hmac = ws->p_hash_hmac_impl;
 
     /* First compute hmac(secret + A(0)) */
     POSIX_GUARD(hmac->init(ws, alg, secret));
@@ -277,7 +282,7 @@ static int s2n_p_hash(struct s2n_prf_working_space *ws, s2n_hmac_algorithm alg,
             POSIX_GUARD(hmac->update(ws, seed_c->data, seed_c->size));
         }
     }
-    POSIX_GUARD(hmac->final(ws, ws->tls.digest0, digest_size));
+    POSIX_GUARD(hmac->final(ws, ws->digest0, digest_size));
 
     uint32_t outputlen = out->size;
     uint8_t *output = out->data;
@@ -285,7 +290,7 @@ static int s2n_p_hash(struct s2n_prf_working_space *ws, s2n_hmac_algorithm alg,
     while (outputlen) {
         /* Now compute hmac(secret + A(N - 1) + seed) */
         POSIX_GUARD(hmac->reset(ws));
-        POSIX_GUARD(hmac->update(ws, ws->tls.digest0, digest_size));
+        POSIX_GUARD(hmac->update(ws, ws->digest0, digest_size));
 
         /* Add the label + seed and compute this round's A */
         POSIX_GUARD(hmac->update(ws, label->data, label->size));
@@ -297,20 +302,20 @@ static int s2n_p_hash(struct s2n_prf_working_space *ws, s2n_hmac_algorithm alg,
             }
         }
 
-        POSIX_GUARD(hmac->final(ws, ws->tls.digest1, digest_size));
+        POSIX_GUARD(hmac->final(ws, ws->digest1, digest_size));
 
         uint32_t bytes_to_xor = MIN(outputlen, digest_size);
 
         for (uint32_t i = 0; i < bytes_to_xor; i++) {
-            *output ^= ws->tls.digest1[i];
+            *output ^= ws->digest1[i];
             output++;
             outputlen--;
         }
 
         /* Stash a digest of A(N), in A(N), for the next round */
         POSIX_GUARD(hmac->reset(ws));
-        POSIX_GUARD(hmac->update(ws, ws->tls.digest0, digest_size));
-        POSIX_GUARD(hmac->final(ws, ws->tls.digest0, digest_size));
+        POSIX_GUARD(hmac->update(ws, ws->digest0, digest_size));
+        POSIX_GUARD(hmac->final(ws, ws->digest0, digest_size));
     }
 
     POSIX_GUARD(hmac->cleanup(ws));
@@ -326,35 +331,68 @@ const struct s2n_p_hash_hmac *s2n_get_hmac_implementation() {
 #endif
 }
 
-int s2n_prf_new(struct s2n_connection *conn)
+S2N_RESULT s2n_prf_new(struct s2n_connection *conn)
 {
-    /* Set p_hash_hmac_impl on initial prf creation. 
-     * When in FIPS mode, the EVP API's must be used for the p_hash HMAC.
-     */
-    conn->prf_space.tls.p_hash_hmac_impl = s2n_get_hmac_implementation();
+    RESULT_ENSURE_REF(conn);
+    RESULT_ENSURE_EQ(conn->prf_space, NULL);
 
-    return conn->prf_space.tls.p_hash_hmac_impl->alloc(&conn->prf_space);
+    DEFER_CLEANUP(struct s2n_blob mem = { 0 }, s2n_free);
+    RESULT_GUARD_POSIX(s2n_realloc(&mem, sizeof(struct s2n_prf_working_space)));
+    RESULT_GUARD_POSIX(s2n_blob_zero(&mem));
+    conn->prf_space = (struct s2n_prf_working_space*)(void*) mem.data;
+    ZERO_TO_DISABLE_DEFER_CLEANUP(mem);
+
+    /* Allocate the hmac state */
+    conn->prf_space->p_hash_hmac_impl = s2n_get_hmac_implementation();
+    RESULT_GUARD_POSIX(conn->prf_space->p_hash_hmac_impl->alloc(conn->prf_space));
+    return S2N_RESULT_OK;
 }
 
-int s2n_prf_free(struct s2n_connection *conn)
+S2N_RESULT s2n_prf_wipe(struct s2n_connection *conn)
 {
-    /* Ensure that p_hash_hmac_impl is set, as it may have been reset for prf_space on s2n_connection_wipe. 
-     * When in FIPS mode, the EVP API's must be used for the p_hash HMAC.
-     */
-    conn->prf_space.tls.p_hash_hmac_impl = s2n_get_hmac_implementation();
+    RESULT_ENSURE_REF(conn);
+    RESULT_ENSURE_REF(conn->prf_space);
 
-    return conn->prf_space.tls.p_hash_hmac_impl->free(&conn->prf_space);
+    /* If we actually initialized s2n_hmac, wipe it.
+     * A valid, initialized s2n_hmac_state will have a valid block size.
+     */
+    if (conn->prf_space->p_hash.s2n_hmac.hash_block_size != 0) {
+        RESULT_GUARD_POSIX(s2n_hmac_reset(&conn->prf_space->p_hash.s2n_hmac));
+    }
+
+    return S2N_RESULT_OK;
+}
+
+S2N_RESULT s2n_prf_free(struct s2n_connection *conn)
+{
+    RESULT_ENSURE_REF(conn);
+    if (conn->prf_space == NULL) {
+        return S2N_RESULT_OK;
+    }
+
+    /* cppcheck-suppress nullPointerRedundantCheck
+     * cppcheck doesn't recognize the early exit if conn->prf_space == NULL.
+     */
+    conn->prf_space->p_hash_hmac_impl = s2n_get_hmac_implementation();
+    RESULT_GUARD_POSIX(conn->prf_space->p_hash_hmac_impl->free(conn->prf_space));
+
+    RESULT_GUARD_POSIX(s2n_free_object((uint8_t **) &conn->prf_space, sizeof(struct s2n_prf_working_space)));
+    return S2N_RESULT_OK;
 }
 
 static int s2n_prf(struct s2n_connection *conn, struct s2n_blob *secret, struct s2n_blob *label, struct s2n_blob *seed_a,
                    struct s2n_blob *seed_b, struct s2n_blob *seed_c, struct s2n_blob *out)
 {
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(secret);
+    POSIX_ENSURE_REF(conn->prf_space);
+
     /* seed_a is always required, seed_b is optional, if seed_c is provided seed_b must also be provided */
     S2N_ERROR_IF(seed_a == NULL, S2N_ERR_PRF_INVALID_SEED);
     S2N_ERROR_IF(seed_b == NULL && seed_c != NULL, S2N_ERR_PRF_INVALID_SEED);
 
     if (conn->actual_protocol_version == S2N_SSLv3) {
-        return s2n_sslv3_prf(&conn->prf_space, secret, seed_a, seed_b, seed_c, out);
+        return s2n_sslv3_prf(conn, secret, seed_a, seed_b, seed_c, out);
     }
 
     /* We zero the out blob because p_hash works by XOR'ing with the existing
@@ -368,18 +406,18 @@ static int s2n_prf(struct s2n_connection *conn, struct s2n_blob *secret, struct
     /* Ensure that p_hash_hmac_impl is set, as it may have been reset for prf_space on s2n_connection_wipe. 
      * When in FIPS mode, the EVP API's must be used for the p_hash HMAC.
      */
-    conn->prf_space.tls.p_hash_hmac_impl = s2n_get_hmac_implementation();
+    conn->prf_space->p_hash_hmac_impl = s2n_get_hmac_implementation();
 
     if (conn->actual_protocol_version == S2N_TLS12) {
-        return s2n_p_hash(&conn->prf_space, conn->secure.cipher_suite->prf_alg, secret, label, seed_a, seed_b,
+        return s2n_p_hash(conn->prf_space, conn->secure.cipher_suite->prf_alg, secret, label, seed_a, seed_b,
                           seed_c, out);
     }
 
     struct s2n_blob half_secret = {.data = secret->data,.size = (secret->size + 1) / 2 };
 
-    POSIX_GUARD(s2n_p_hash(&conn->prf_space, S2N_HMAC_MD5, &half_secret, label, seed_a, seed_b, seed_c, out));
+    POSIX_GUARD(s2n_p_hash(conn->prf_space, S2N_HMAC_MD5, &half_secret, label, seed_a, seed_b, seed_c, out));
     half_secret.data += secret->size - half_secret.size;
-    POSIX_GUARD(s2n_p_hash(&conn->prf_space, S2N_HMAC_SHA1, &half_secret, label, seed_a, seed_b, seed_c, out));
+    POSIX_GUARD(s2n_p_hash(conn->prf_space, S2N_HMAC_SHA1, &half_secret, label, seed_a, seed_b, seed_c, out));
 
     return 0;
 }
@@ -408,6 +446,47 @@ int s2n_hybrid_prf_master_secret(struct s2n_connection *conn, struct s2n_blob *p
     return s2n_prf(conn, premaster_secret, &label, &client_random, &server_random, &conn->kex_params.client_key_exchange_message, &master_secret);
 }
 
+int s2n_prf_calculate_master_secret(struct s2n_connection *conn, struct s2n_blob *premaster_secret)
+{
+    POSIX_ENSURE_REF(conn);
+
+    POSIX_ENSURE_EQ(s2n_conn_get_current_message_type(conn), CLIENT_KEY);
+
+    if(!conn->ems_negotiated) {
+        POSIX_GUARD(s2n_tls_prf_master_secret(conn, premaster_secret));
+        return S2N_SUCCESS;
+    }
+
+    /* Only the client writes the Client Key Exchange message */
+    if (conn->mode == S2N_CLIENT) {
+        POSIX_GUARD(s2n_handshake_finish_header(&conn->handshake.io));
+    }
+    struct s2n_stuffer client_key_message = conn->handshake.io;
+    POSIX_GUARD(s2n_stuffer_reread(&client_key_message));
+    uint32_t client_key_message_size = s2n_stuffer_data_available(&client_key_message);
+    struct s2n_blob client_key_blob = { 0 };
+    POSIX_GUARD(s2n_blob_init(&client_key_blob, client_key_message.blob.data, client_key_message_size));
+
+    uint8_t data[S2N_MAX_DIGEST_LEN] = { 0 };
+    struct s2n_blob digest = { 0 };
+    POSIX_GUARD(s2n_blob_init(&digest, data, sizeof(data)));
+    if (conn->actual_protocol_version < S2N_TLS12) {
+        uint8_t sha1_data[S2N_MAX_DIGEST_LEN] = { 0 };
+        struct s2n_blob sha1_digest = { 0 };
+        POSIX_GUARD(s2n_blob_init(&sha1_digest, sha1_data, sizeof(sha1_data)));
+        POSIX_GUARD_RESULT(s2n_prf_get_digest_for_ems(conn, &client_key_blob, S2N_HASH_MD5, &digest));
+        POSIX_GUARD_RESULT(s2n_prf_get_digest_for_ems(conn, &client_key_blob, S2N_HASH_SHA1, &sha1_digest));
+        POSIX_GUARD_RESULT(s2n_tls_prf_extended_master_secret(conn, premaster_secret, &digest, &sha1_digest));
+    } else {
+        s2n_hmac_algorithm prf_alg = conn->secure.cipher_suite->prf_alg;
+        s2n_hash_algorithm hash_alg = 0;
+        POSIX_GUARD(s2n_hmac_hash_alg(prf_alg, &hash_alg));
+        POSIX_GUARD_RESULT(s2n_prf_get_digest_for_ems(conn, &client_key_blob, hash_alg, &digest));
+        POSIX_GUARD_RESULT(s2n_tls_prf_extended_master_secret(conn, premaster_secret, &digest, NULL));
+    }
+    return S2N_SUCCESS;
+}
+
 /**
  *= https://tools.ietf.org/rfc/rfc7627#section-4
  *# When the extended master secret extension is negotiated in a full
@@ -417,7 +496,7 @@ int s2n_hybrid_prf_master_secret(struct s2n_connection *conn, struct s2n_blob *p
  *#                    session_hash)
  *#                    [0..47];
  */
-S2N_RESULT s2n_tls_prf_extended_master_secret(struct s2n_connection *conn, struct s2n_blob *premaster_secret, struct s2n_blob *session_hash)
+S2N_RESULT s2n_tls_prf_extended_master_secret(struct s2n_connection *conn, struct s2n_blob *premaster_secret, struct s2n_blob *session_hash, struct s2n_blob *sha1_hash)
 {
     struct s2n_blob extended_master_secret = {.size = sizeof(conn->secrets.master_secret), .data = conn->secrets.master_secret};
 
@@ -425,34 +504,36 @@ S2N_RESULT s2n_tls_prf_extended_master_secret(struct s2n_connection *conn, struc
     /* Subtract one from the label size to remove the "\0" */
     struct s2n_blob label = {.size = sizeof(extended_master_secret_label) - 1, .data = extended_master_secret_label};
 
-    RESULT_GUARD_POSIX(s2n_prf(conn, premaster_secret, &label, session_hash, NULL, NULL, &extended_master_secret));
+    RESULT_GUARD_POSIX(s2n_prf(conn, premaster_secret, &label, session_hash, sha1_hash, NULL, &extended_master_secret));
 
     return S2N_RESULT_OK;
 }
 
-S2N_RESULT s2n_prf_get_digest_for_ems(struct s2n_connection *conn, struct s2n_blob *message, struct s2n_blob *output)
+S2N_RESULT s2n_prf_get_digest_for_ems(struct s2n_connection *conn, struct s2n_blob *message, s2n_hash_algorithm hash_alg, struct s2n_blob *output)
 {
     RESULT_ENSURE_REF(conn);
+    RESULT_ENSURE_REF(conn->handshake.hashes);
     RESULT_ENSURE_REF(output);
 
     struct s2n_hash_state hash_state = { 0 };
-    s2n_hmac_algorithm prf_alg = conn->secure.cipher_suite->prf_alg;
-    s2n_hash_algorithm hash_alg = 0;
-    RESULT_GUARD_POSIX(s2n_hmac_hash_alg(prf_alg, &hash_alg));
     RESULT_GUARD_POSIX(s2n_handshake_get_hash_state(conn, hash_alg, &hash_state));
-    RESULT_GUARD_POSIX(s2n_hash_copy(&conn->hash_workspace, &hash_state));
-    RESULT_GUARD_POSIX(s2n_hash_update(&conn->hash_workspace, message->data, message->size));
+    RESULT_GUARD_POSIX(s2n_hash_copy(&conn->handshake.hashes->hash_workspace, &hash_state));
+    RESULT_GUARD_POSIX(s2n_hash_update(&conn->handshake.hashes->hash_workspace, message->data, message->size));
 
     uint8_t digest_size = 0;
-    RESULT_GUARD_POSIX(s2n_hash_digest_size(conn->hash_workspace.alg, &digest_size));
+    RESULT_GUARD_POSIX(s2n_hash_digest_size(conn->handshake.hashes->hash_workspace.alg, &digest_size));
     RESULT_ENSURE_GTE(output->size, digest_size);
-    RESULT_GUARD_POSIX(s2n_hash_digest(&conn->hash_workspace, output->data, digest_size));
+    RESULT_GUARD_POSIX(s2n_hash_digest(&conn->handshake.hashes->hash_workspace, output->data, digest_size));
+    output->size = digest_size;
 
     return S2N_RESULT_OK;
 }
 
 static int s2n_sslv3_finished(struct s2n_connection *conn, uint8_t prefix[4], struct s2n_hash_state *hash_workspace, uint8_t * out)
 {
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(conn->handshake.hashes);
+
     uint8_t xorpad1[48] =
         { 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36,
         0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36
@@ -467,7 +548,7 @@ static int s2n_sslv3_finished(struct s2n_connection *conn, uint8_t prefix[4], st
     POSIX_ENSURE_LTE(MD5_DIGEST_LENGTH + SHA_DIGEST_LENGTH, sizeof(conn->handshake.client_finished));
 
     struct s2n_hash_state *md5 = hash_workspace;
-    POSIX_GUARD(s2n_hash_copy(md5, &conn->handshake.md5));
+    POSIX_GUARD(s2n_hash_copy(md5, &conn->handshake.hashes->md5));
     POSIX_GUARD(s2n_hash_update(md5, prefix, 4));
     POSIX_GUARD(s2n_hash_update(md5, conn->secrets.master_secret, sizeof(conn->secrets.master_secret)));
     POSIX_GUARD(s2n_hash_update(md5, xorpad1, 48));
@@ -480,7 +561,7 @@ static int s2n_sslv3_finished(struct s2n_connection *conn, uint8_t prefix[4], st
     POSIX_GUARD(s2n_hash_reset(md5));
 
     struct s2n_hash_state *sha1 = hash_workspace;
-    POSIX_GUARD(s2n_hash_copy(sha1, &conn->handshake.sha1));
+    POSIX_GUARD(s2n_hash_copy(sha1, &conn->handshake.hashes->sha1));
     POSIX_GUARD(s2n_hash_update(sha1, prefix, 4));
     POSIX_GUARD(s2n_hash_update(sha1, conn->secrets.master_secret, sizeof(conn->secrets.master_secret)));
     POSIX_GUARD(s2n_hash_update(sha1, xorpad1, 40));
@@ -497,22 +578,31 @@ static int s2n_sslv3_finished(struct s2n_connection *conn, uint8_t prefix[4], st
 
 static int s2n_sslv3_client_finished(struct s2n_connection *conn)
 {
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(conn->handshake.hashes);
+
     uint8_t prefix[4] = { 0x43, 0x4c, 0x4e, 0x54 };
 
     POSIX_ENSURE_LTE(MD5_DIGEST_LENGTH + SHA_DIGEST_LENGTH, sizeof(conn->handshake.client_finished));
-    return s2n_sslv3_finished(conn, prefix, &conn->hash_workspace, conn->handshake.client_finished);
+    return s2n_sslv3_finished(conn, prefix, &conn->handshake.hashes->hash_workspace, conn->handshake.client_finished);
 }
 
 static int s2n_sslv3_server_finished(struct s2n_connection *conn)
 {
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(conn->handshake.hashes);
+
     uint8_t prefix[4] = { 0x53, 0x52, 0x56, 0x52 };
 
     POSIX_ENSURE_LTE(MD5_DIGEST_LENGTH + SHA_DIGEST_LENGTH, sizeof(conn->handshake.server_finished));
-    return s2n_sslv3_finished(conn, prefix, &conn->hash_workspace, conn->handshake.server_finished);
+    return s2n_sslv3_finished(conn, prefix, &conn->handshake.hashes->hash_workspace, conn->handshake.server_finished);
 }
 
 int s2n_prf_client_finished(struct s2n_connection *conn)
 {
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(conn->handshake.hashes);
+
     struct s2n_blob master_secret, md5, sha;
     uint8_t md5_digest[MD5_DIGEST_LENGTH];
     uint8_t sha_digest[SHA384_DIGEST_LENGTH];
@@ -534,13 +624,13 @@ int s2n_prf_client_finished(struct s2n_connection *conn)
     if (conn->actual_protocol_version == S2N_TLS12) {
         switch (conn->secure.cipher_suite->prf_alg) {
         case S2N_HMAC_SHA256:
-            POSIX_GUARD(s2n_hash_copy(&conn->hash_workspace, &conn->handshake.sha256));
-            POSIX_GUARD(s2n_hash_digest(&conn->hash_workspace, sha_digest, SHA256_DIGEST_LENGTH));
+            POSIX_GUARD(s2n_hash_copy(&conn->handshake.hashes->hash_workspace, &conn->handshake.hashes->sha256));
+            POSIX_GUARD(s2n_hash_digest(&conn->handshake.hashes->hash_workspace, sha_digest, SHA256_DIGEST_LENGTH));
             sha.size = SHA256_DIGEST_LENGTH;
             break;
         case S2N_HMAC_SHA384:
-            POSIX_GUARD(s2n_hash_copy(&conn->hash_workspace, &conn->handshake.sha384));
-            POSIX_GUARD(s2n_hash_digest(&conn->hash_workspace, sha_digest, SHA384_DIGEST_LENGTH));
+            POSIX_GUARD(s2n_hash_copy(&conn->handshake.hashes->hash_workspace, &conn->handshake.hashes->sha384));
+            POSIX_GUARD(s2n_hash_digest(&conn->handshake.hashes->hash_workspace, sha_digest, SHA384_DIGEST_LENGTH));
             sha.size = SHA384_DIGEST_LENGTH;
             break;
         default:
@@ -551,13 +641,13 @@ int s2n_prf_client_finished(struct s2n_connection *conn)
         return s2n_prf(conn, &master_secret, &label, &sha, NULL, NULL, &client_finished);
     }
 
-    POSIX_GUARD(s2n_hash_copy(&conn->hash_workspace, &conn->handshake.md5));
-    POSIX_GUARD(s2n_hash_digest(&conn->hash_workspace, md5_digest, MD5_DIGEST_LENGTH));
+    POSIX_GUARD(s2n_hash_copy(&conn->handshake.hashes->hash_workspace, &conn->handshake.hashes->md5));
+    POSIX_GUARD(s2n_hash_digest(&conn->handshake.hashes->hash_workspace, md5_digest, MD5_DIGEST_LENGTH));
     md5.data = md5_digest;
     md5.size = MD5_DIGEST_LENGTH;
 
-    POSIX_GUARD(s2n_hash_copy(&conn->hash_workspace, &conn->handshake.sha1));
-    POSIX_GUARD(s2n_hash_digest(&conn->hash_workspace, sha_digest, SHA_DIGEST_LENGTH));
+    POSIX_GUARD(s2n_hash_copy(&conn->handshake.hashes->hash_workspace, &conn->handshake.hashes->sha1));
+    POSIX_GUARD(s2n_hash_digest(&conn->handshake.hashes->hash_workspace, sha_digest, SHA_DIGEST_LENGTH));
     sha.data = sha_digest;
     sha.size = SHA_DIGEST_LENGTH;
 
@@ -566,6 +656,9 @@ int s2n_prf_client_finished(struct s2n_connection *conn)
 
 int s2n_prf_server_finished(struct s2n_connection *conn)
 {
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(conn->handshake.hashes);
+
     struct s2n_blob master_secret, md5, sha;
     uint8_t md5_digest[MD5_DIGEST_LENGTH];
     uint8_t sha_digest[SHA384_DIGEST_LENGTH];
@@ -587,13 +680,13 @@ int s2n_prf_server_finished(struct s2n_connection *conn)
     if (conn->actual_protocol_version == S2N_TLS12) {
         switch (conn->secure.cipher_suite->prf_alg) {
         case S2N_HMAC_SHA256:
-            POSIX_GUARD(s2n_hash_copy(&conn->hash_workspace, &conn->handshake.sha256));
-            POSIX_GUARD(s2n_hash_digest(&conn->hash_workspace, sha_digest, SHA256_DIGEST_LENGTH));
+            POSIX_GUARD(s2n_hash_copy(&conn->handshake.hashes->hash_workspace, &conn->handshake.hashes->sha256));
+            POSIX_GUARD(s2n_hash_digest(&conn->handshake.hashes->hash_workspace, sha_digest, SHA256_DIGEST_LENGTH));
             sha.size = SHA256_DIGEST_LENGTH;
             break;
         case S2N_HMAC_SHA384:
-            POSIX_GUARD(s2n_hash_copy(&conn->hash_workspace, &conn->handshake.sha384));
-            POSIX_GUARD(s2n_hash_digest(&conn->hash_workspace, sha_digest, SHA384_DIGEST_LENGTH));
+            POSIX_GUARD(s2n_hash_copy(&conn->handshake.hashes->hash_workspace, &conn->handshake.hashes->sha384));
+            POSIX_GUARD(s2n_hash_digest(&conn->handshake.hashes->hash_workspace, sha_digest, SHA384_DIGEST_LENGTH));
             sha.size = SHA384_DIGEST_LENGTH;
             break;
         default:
@@ -604,13 +697,13 @@ int s2n_prf_server_finished(struct s2n_connection *conn)
         return s2n_prf(conn, &master_secret, &label, &sha, NULL, NULL, &server_finished);
     }
 
-    POSIX_GUARD(s2n_hash_copy(&conn->hash_workspace, &conn->handshake.md5));
-    POSIX_GUARD(s2n_hash_digest(&conn->hash_workspace, md5_digest, MD5_DIGEST_LENGTH));
+    POSIX_GUARD(s2n_hash_copy(&conn->handshake.hashes->hash_workspace, &conn->handshake.hashes->md5));
+    POSIX_GUARD(s2n_hash_digest(&conn->handshake.hashes->hash_workspace, md5_digest, MD5_DIGEST_LENGTH));
     md5.data = md5_digest;
     md5.size = MD5_DIGEST_LENGTH;
 
-    POSIX_GUARD(s2n_hash_copy(&conn->hash_workspace, &conn->handshake.sha1));
-    POSIX_GUARD(s2n_hash_digest(&conn->hash_workspace, sha_digest, SHA_DIGEST_LENGTH));
+    POSIX_GUARD(s2n_hash_copy(&conn->handshake.hashes->hash_workspace, &conn->handshake.hashes->sha1));
+    POSIX_GUARD(s2n_hash_digest(&conn->handshake.hashes->hash_workspace, sha_digest, SHA_DIGEST_LENGTH));
     sha.data = sha_digest;
     sha.size = SHA_DIGEST_LENGTH;
 
