@@ -650,7 +650,7 @@ static void s_aws_input_stream_py_destroy(struct aws_input_stream *stream) {
 
 static int s_aws_input_stream_py_seek(
     struct aws_input_stream *stream,
-    aws_off_t offset,
+    int64_t offset,
     enum aws_stream_seek_basis basis) {
 
     struct aws_input_stream_py_impl *impl = stream->impl;
@@ -664,7 +664,7 @@ static int s_aws_input_stream_py_seek(
         return AWS_OP_ERR; /* Python has shut down. Nothing matters anymore, but don't crash */
     }
 
-    method_result = PyObject_CallMethod(impl->self_proxy, "_seek", "(li)", offset, basis);
+    method_result = PyObject_CallMethod(impl->self_proxy, "_seek", "(Li)", offset, basis);
     if (!method_result) {
         aws_result = aws_py_raise_error();
         goto done;
